#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

/*
 * This program is a small calculator
 *
 * The user can enter an operation, possibly followed by a value
 * for example: +5 will add 5 to the accumulator.
 *
 * Since the operation is in buf[0], the value must be somewhere
 * behind it.  The soonest it can be is starting in buf[1]
 * Since atoi() and atof() ignore leading spaces,
 * we can then use atof(buf+1) to get the value.
 */

void do_help(void); /* prototype for the do_help routine */

main()
{
    char buf[BUFSIZ];   /* input area for user */
    float acc, num;     /* accumulator and number */

    acc = 0.0;          /* start with accumulator at zero */

    do
    { /* loop until done */
        printf("Enter operation: ");
        gets(buf);  /* operator and possibly a value */
        switch (buf[0])
        { /* process operation */
        case '+':   /* add value */
            num = atof(buf+1);  /* get number to add */
            acc += num;
            break;
        case '-':   /* subtract */
            num = atof(buf+1);  /* value follows the operator */
            acc -= num;
            break;
        case '*':   /* multiply */
            num = atof(buf+1);
            acc *= num;
            break;
        case '/':   /* divide */
            num = atof(buf+1);
            if (num != 0.0) acc /= num;
            break;
        case 's':   /* square or square root */
            /* examine the second character to see which command */
            if (strcmp(buf, "sq") == 0)
            { /* square the accumulator */
                acc = acc * acc;    /* square accumulator */
            } else if (strcmp(buf, "sr") == 0)
            {
                acc = sqrt(acc);    /* take square root */
            }
            else
            { /* invalid operator */
                printf("Invalid operator (%s)\n", buf);
            } /* end of check for second character */
        	/* end of case for the first character */
        	break;

        case 'q':   /* quit */
            break;
        case 'c':   /* clear accumulator */
            acc = 0.0;
            break;
        case 'h':   /* help message */
            do_help();
            break;
        default:    /* unknown command */
            printf("Invalid operation (%s)\n", buf);
        }
        printf("%f\n", acc);
    } while (buf[0] != 'q');

    printf("Press ENTER to terminate program: "); /* display another message */
	getchar(); 						/* wait for the user to press ENTER */
 }

 /* display a help message */
 void do_help(void)
 {
    printf("This program is a small calculator\n");
    printf("Enter an operation followed by a value if required\n");
    printf("For example:\n");
    printf("\t+ 123.45\n");
    printf("will add the value 123.45 to the accumulator\n");
    printf("Supported operations are:\n");
    printf("+ Add the value\n");
    printf("- Subtract the value\n");
    printf("* Multiply by the value\n");
    printf("/ Divide by the value\n");
    printf("sq Square the accumulator\n");
    printf("sr Take the square root of the accumulator\n");
    printf("c Clear the accumulator (set it to zero)\n");
    printf("h Display this message\n");
    printf("q Quit the program\n");
 }
