#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 * This program extends the getinfo.c program
 * Instead of displaying the customers at the end of the program,
 * it writes the information out to a file
 *
 * It will also read the information in from a file if desired
 */

#include "getinfo.h"

/* function prototypes */
int add_cust(struct cust cust[], int nent);
void display_cust(struct cust cust[], int nent);
void write_cust(struct cust cust[], int nent, FILE *fp);
int read_cust(struct cust cust[], int nent, FILE *fp);
int read_line(char buf[], FILE *fp);

main()
{
    char buf[BUFSIZ];
    struct cust Cust[MAXENT];
    int ncust=0;    /* number of customers */
    int i;          /* loop variable */
    int ret;        /* return value from add_cust */
    FILE *fp;       /* file for saving records */

    /* read information in from file if needed */
    printf("Enter file name to read from, or ENTER: ");
    gets(buf);
    if (buf[0] != '\0')
    { /* a file name was given - open that file and read the info */
        if ((fp = fopen(buf, "r")) == NULL)
        { /* couldn't open file - complain */
            fprintf(stderr, "Can't open file (%s)\n", buf);
            /* the exit function terminates the program */
            exit(1);    /* quit program */
        }
        /* loop reading records until done */
        for (ncust = 0; ncust < MAXENT; ncust++)
        { /* read records in from file */
            ret = read_cust(Cust, ncust, fp);
            if (ret == ADD_DONE) break; /* done reading */
            display_cust(Cust, ncust);  /* display this entry */
        }
        /* done reading - close file */
        fclose(fp);
    }

    /* get information from the user and store it into an array */
    while (1)
    { /* loop getting information until done adding */
        if ((ret=add_cust(Cust, ncust)) == ADD_DONE) break;   /* done adding */
        if (ret != ADD_OK) continue;    /* bad add - ignore */
        /* entry looks ok - ask user to accept or reject record */
        display_cust(Cust, ncust);  /* display new customer */
        printf("Ok? (y or n): ");   /* ask if okay */
        gets(buf);
        if (buf[0] == 'n') continue;    /* not okay - try again */
        /* new entry accepted */
        ncust++;    /* one more customer in table */
    }

    /* all customers have been added, write entries out to file if desired */
    printf("Enter name of output file, or ENTER to not save: ");
    gets(buf);
    if (buf[0] == '\0') exit(0);    /* don't save - just exit */

    /* file name given - write to it */
    fp = fopen(buf, "w");
    if (fp == NULL)
    {
    	fprintf(stderr, "Can't open output file (%s)\n", buf);
        exit(2);    /* fatal error - quit */
    }

    /* output file open - write out records */
    for (i=0; i < ncust; i++)
    { /* loop through array of customers */
        write_cust(Cust, i, fp);
    }
    /* all done - close file */
    fclose(fp);

    printf("Press ENTER to terminate program: "); /* display another message */
	getchar(); 						/* wait for the user to press ENTER */
 }

/*
 * This function adds a customer to the table
 * If the user adds a customer, the function returns a zero
 * If the user is done, the function returns a non-zero value
 */
int add_cust(struct cust cust[], int ncust)
{
    char buf[BUFSIZ];

    printf("Enter customer ID or ENTER to quit: ");
    gets(buf);
    if (buf[0] == '\0') return ADD_DONE;   /* don't add */
    cust[ncust].id = atoi(buf);
    if (cust[ncust].id <= 0) return ADD_BAD;  /* invalid ID - reject entry */

    do
    { /* get last name */
		printf("Enter Last Name: ");
        gets(buf);
        /* if name isn't too long, copy it into structure */
        if (strlen(buf) < NAMESIZ) strcpy(cust[ncust].lname, buf);
    } while (strlen(buf) >= NAMESIZ);   /* keep looping until valid */

    do
    { /* get first name */
    	printf("Enter First Name: ");
    	gets(buf);
        if (strlen(buf) < NAMESIZ) strcpy(cust[ncust].fname, buf);
    } while (strlen(buf) >= NAMESIZ); /* keep looping until valid */

    /* now get balance */
    printf("Enter balance: ");
    gets(buf);
    cust[ncust].bal = atof(buf);    /* convert age to number */

    /* now increment how many customers we have */
    return ADD_OK;  /* valid add */
 }

void display_cust(struct cust cust[], int ent)
{
    printf("Customer %d: %s %s, bal %.2f\n", cust[ent].id, cust[ent].fname,
    		cust[ent].lname, cust[ent].bal);
}

/*
 * This function reads the next entry file the file
 * If the entry is valid, ADD_OK is returned.
 * If any problems are found, ADD_BAD is returned
 * At end of file, ADD_DONE is returned
 *
 * If we encounter a problem, we still have the read the entire entry
 * otherwise, the next read will be out of synch with the file
 *
 * If we encounter and end of file, we return ADD_DONE immediately
 * since there is nothing more to read from the file
 * We should only get this on the first read (customer ID), but
 * we test it each time just in case.
 *
 */
int read_cust(struct cust cust[], int ent, FILE *fp)
{ /* read entry from file into cust table */
    char buf[BUFSIZ];
    int ret = ADD_OK;

    /* first line is customer ID */
    if (read_line(buf, fp) == ADD_DONE) return ADD_DONE;    /* end of file */
    cust[ent].id = atoi(buf);   /* convert ID to number */
    if (cust[ent].id <= 0) ret = ADD_BAD;  /* reject entry */

    /* second line is last name */
    if (read_line(buf, fp) == ADD_DONE) return ADD_DONE;
    if (strlen(buf) < NAMESIZ) strcpy(cust[ent].lname, buf);
    else ret = ADD_BAD;

    /* next is first name */
    if (read_line(buf, fp) == ADD_DONE) return ADD_DONE;
    if (strlen(buf) < NAMESIZ) strcpy(cust[ent].fname, buf);
    else ret = ADD_BAD;

    /* last line is balance */
    if (read_line(buf, fp) == ADD_DONE) return ADD_DONE;
    cust[ent].bal = atof(buf);

    return ret;
}


int read_line(char buf[], FILE *fp)
{ /* read line from file and strip 'end-of-line' sequence */
    int i;

    /*
     * read next line from file
     *
     * If end of file encountered, return ADD_DONE
     * otherwise, return ADD_OK
     */
    if (fgets(buf, BUFSIZ, fp)==NULL) return ADD_DONE;
    /* find end of line - CR, LF, or binary zero */
    for (i=0; buf[i] != '\r' && buf[i] != '\n' & buf[i] != '\0'; i++);
    buf[i] = '\0';  /* truncate string */
    return ADD_OK;
}

void write_cust(struct cust cust[], int ent, FILE *fp)
{ /* write customer record out to file */
    /* write one field per line to make it easy to read back in later */
    fprintf(fp, "%d\n%s\n%s\n%f\n", cust[ent].id, cust[ent].lname,
                cust[ent].fname, cust[ent].bal);
}
