/*
 * This is the header file for the getinfo program
 */

#define MAXENT 100

/*
 * The customer structure is different than previous versions
 * rather than specifying a maximum length of the name field,
 * we simply define a character pointer.
 * This means that there is no limit to the size of the names
 */
struct cust
{
    int id;                 /* customer number */
    char *lname;			/* last name of customer */
    char *fname;			/* first name of customer */
    float bal;              /* balance of customer */
};

/* the next structure isn't used until random I/O */
struct index
{
    int id;                 /* customer number */
    long loc;               /* location in file */
};

/* function prototypes */
void display_menu(void);    /* display menu for user */
struct cust *get_cust(void);    /* get customer info from user */
struct cust *read_cust(struct index *ndx, FILE *fp);   /* read 'ent' record from file */
void write_cust(struct index *ndx, struct cust *cust, FILE *fp); /* write record to file */
int find_cust(struct index[], int id);  /* find customer ID in index field */
void display_cust(struct cust *cust);   /* display cust record */
void list_cust(struct index *ndx, FILE *fp);	/* list all customers */
void edit_cust(struct cust *cust);	/* allow user to edit cust rec */
struct cust *alloc_cust(void);  /* allocate memory for customer record */
void free_cust(struct cust *cust);  /* free memory for cust rec */
int read_line(char buf[], FILE *fp);
void search_lname(struct index *ndx, char *lname, FILE *fp); /* display all cust with this lname */
void compress_file(struct index *ndx, FILE *fp); /* remove deleted records */
void do_help(void); /* display help message for user */

